package cn.com.qmth.stmms.biz.mark.service.Impl;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.mark.model.ArbitrateHistory;
import cn.com.qmth.stmms.biz.mark.model.ArbitrationDTO;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.MarkSpecialTag;
import cn.com.qmth.stmms.biz.mark.model.MarkStepDTO;
import cn.com.qmth.stmms.biz.mark.model.MarkTrack;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.biz.mark.model.SpecialTagDTO;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.model.TrackDTO;
import cn.com.qmth.stmms.biz.mark.model.TrialHistory;
import cn.com.qmth.stmms.biz.mark.model.TrialLibrary;
import cn.com.qmth.stmms.biz.mark.model.TrialTag;
import cn.com.qmth.stmms.biz.mark.model.TrialTrack;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.biz.mark.service.TrialService;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LibraryStatus;

/**
 * 评卷任务服务实现
 *
 * @author LS
 */
@Service("taskService")
public class TaskServiceImpl implements TaskService {

    private static Logger log = LoggerFactory.getLogger(TaskServiceImpl.class);

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService trackService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private TrialService trialService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private UserService userService;

    @Autowired
    private FileService fileService;

    @Autowired
    private ExamService examService;

    @Override
    public List<Task> findByQuery(MarkLibrarySearchQuery query) {
        List<Task> list = new LinkedList<Task>();
        query = libraryService.findByQuery(query);
        if (query.getCurrentCount() > 0) {
            for (MarkLibrary library : query.getResult()) {
                list.add(build(library));
            }
        }
        return list;
    }

    @Override
    public Task build(ArbitrateHistory history, MarkGroup group) {
        ExamStudent student = studentService.findByExamIdAndExamNumber(history.getExamId(), history.getExamNumber());
        List<MarkLibrary> libraryList = libraryService.findByStudentAndGroup(student.getId(), group.getNumber());
        Task task = new Task();
        task.setStudentId(history.getStudentId());
        task.setLibraryId(history.getId());
        task.setSecretNumber(history.getSecretNumber());
        task.setQuestionList(buildMarkStep(group, null));
        task.setSliceUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                student.getSliceCount()));
        task.setSliceConfig(group.getPictureConfigList());
        task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        task.setMarkTime(history.getUpdateTime());
        // 构造仲裁信息
        List<ArbitrationDTO> list = new ArrayList<ArbitrationDTO>();
        for (MarkLibrary library : libraryList) {
            if (library.getStatus() != LibraryStatus.WAITING) {
                Marker marker = markerService.findById(library.getMarkerId());
                marker.setUser(userService.findById(marker.getUserId()));
                list.add(new ArbitrationDTO(library, marker));
            }
        }
        // task.setArbitrationList(list);
        // 回评状态下的打分信息
        // if (history.getTotalScore() != null) {
        // task.setTotalScore(history.getTotalScore());
        // }
        // if (history.getScoreList() != null) {
        // task.setScoreList(history.getScoreList());
        // }
        return task;
    }

    @Override
    public Task build(MarkLibrary library) {
        Exam exam = examService.findById(library.getExamId());
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setLibraryId(library.getId());
        task.setStudentId(library.getStudentId());
        task.setSecretNumber(library.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(library.getExamNumber());
        task.setSubject(subject);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSliceConfig(group.getPictureConfigList());
            task.setSheetUrls(fileService.getSheetUris(library.getExamId(), library.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        task.setQuestionList(buildMarkStep(group, library));
        task.setSpecialTagList(getMarkSpecialTagList(library.getId()));
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        task.setMarkTime(library.getMarkerTime());
        if (library.getStatus() == LibraryStatus.REJECTED && library.getMarkerId() != null) {
            task.setRejected(true);
        }
        if (library.getStatus() == LibraryStatus.MARKED || library.getStatus() == LibraryStatus.INSPECTED) {
            task.setMarkerScore(library.getMarkerScore());
            task.setPrevious(true);
        }
        return task;
    }

    @Override
    public Task build(TrialLibrary library, TrialHistory history) {
        Exam exam = examService.findById(library.getExamId());
        ExamSubject subject = subjectService.find(library.getExamId(), library.getSubjectCode());
        ExamStudent student = studentService.findByExamIdAndExamNumber(library.getExamId(), library.getExamNumber());
        MarkGroup group = groupService.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());

        Task task = new Task();
        task.setLibraryId(library.getId());
        task.setStudentId(library.getStudentId());
        task.setSecretNumber(library.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(library.getExamNumber());
        task.setSubject(subject);
        task.setQuestionList(buildTrialStep(group, history));
        task.setObjectiveScore(student.getObjectiveScore());
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(library.getExamId(), library.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(library.getExamId(), library.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSliceConfig(group.getPictureConfigList());
            task.setSheetUrls(fileService.getSheetUris(library.getExamId(), library.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        if (history != null) {
            task.setSpecialTagList(getTrialTagList(history));
            task.setMarkTime(history.getMarkerTime());
            task.setMarkerScore(history.getMarkerScore());
            task.setRejected(true);
        }
        return task;
    }

    private List<MarkStepDTO> buildMarkStep(MarkGroup group, MarkLibrary library) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        List<MarkTrack> tracks = trackService.findByLibraryId(library.getId());
        List<ScoreItem> sItems = library.getScoreList();
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                step.setScore(sItems.get(i).getScore());
            }
            if (library != null) {
                // 增加阅卷轨迹列表获取
                String questionNumber = question.getQuestionNumber();
                for (MarkTrack track : tracks) {
                    if (track.getQuestionNumber().equals(questionNumber)) {
                        step.addTrack(new TrackDTO(track));
                    }
                }
            }
            list.add(step);
        }
        return list;
    }

    private List<MarkStepDTO> buildTrialStep(MarkGroup group, TrialHistory history) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        List<TrialTrack> tracks = new ArrayList<TrialTrack>();
        List<ScoreItem> sItems = new ArrayList<ScoreItem>();
        if (history != null) {
            tracks = trialService.findTrack(history.getLibraryId(), history.getMarkerId());
            sItems = history.getScoreList();
        }
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                step.setScore(sItems.get(i).getScore());
            }
            // 增加阅卷轨迹列表获取
            String questionNumber = question.getQuestionNumber();
            for (TrialTrack track : tracks) {
                if (track.getQuestionNumber().equals(questionNumber)) {
                    step.addTrack(new TrackDTO(track));
                }
            }
            list.add(step);
        }
        return list;
    }

    private MarkStepDTO buildStep(ExamQuestion question) {
        MarkStepDTO step = new MarkStepDTO();
        step.setMainNumber(question.getMainNumber());
        step.setSubNumber(question.getSubNumber());
        step.setGroupNumber(question.getGroupNumber());
        step.setTitle(question.getMainTitle());
        step.setDefaultScore(0d);
        step.setMax(question.getTotalScore());
        step.setMin(0d);
        step.setInterval(question.getIntervalScore());
        return step;
    }

    public SpecialTagDTO[] getMarkSpecialTagList(Integer libraryId) {
        SpecialTagDTO[] specialTags = null;
        List<MarkSpecialTag> list = markSpecialTagService.findByLibraryId(libraryId);
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO();
                specialTagDTO.setTagName(list.get(i).getTagName());
                specialTagDTO.setPositionX(list.get(i).getPositionX());
                specialTagDTO.setPositionY(list.get(i).getPositionY());
                specialTagDTO.setOffsetIndex(list.get(i).getOffsetIndex());
                specialTagDTO.setOffsetX(list.get(i).getOffsetX());
                specialTagDTO.setOffsetY(list.get(i).getOffsetY());
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    public SpecialTagDTO[] getTrialTagList(TrialHistory history) {
        SpecialTagDTO[] specialTags = null;
        List<TrialTag> list = trialService.findTag(history.getLibraryId(), history.getMarkerId());
        if (!list.isEmpty()) {
            specialTags = new SpecialTagDTO[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SpecialTagDTO specialTagDTO = new SpecialTagDTO();
                specialTagDTO.setTagName(list.get(i).getContent());
                specialTagDTO.setPositionX(list.get(i).getPositionX());
                specialTagDTO.setPositionY(list.get(i).getPositionY());
                specialTagDTO.setOffsetIndex(list.get(i).getOffsetIndex());
                specialTagDTO.setOffsetX(list.get(i).getOffsetX());
                specialTagDTO.setOffsetY(list.get(i).getOffsetY());
                specialTags[i] = specialTagDTO;
            }
        }
        return specialTags;
    }

    @Override
    public Task build(Integer studentId) {
        ExamStudent student = studentService.findById(studentId);
        ExamSubject subject = subjectService.find(student.getExamId(), student.getSubjectCode());
        Exam exam = examService.findById(student.getExamId());

        Task task = new Task();
        task.setStudentId(studentId);
        task.setSecretNumber(student.getSecretNumber());
        task.setStudentCode(student.getStudentCode());
        task.setStudentName(student.getName());
        task.setExamNumber(student.getExamNumber());
        task.setSubject(subject);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            task.setJsonUrl(fileService.getJsonUri(student.getExamId(), student.getSecretNumber()));
        } else {
            task.setSliceUrls(fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1,
                    student.getSliceCount()));
            task.setSheetUrls(fileService.getSheetUris(student.getExamId(), student.getExamNumber(), 1,
                    student.getSheetCount()));
        }
        task.setQuestionList(buildMarkStep(student));
        // task.setSpecialTagList(getMarkSpecialTagList(library.getId()));
        task.setObjectiveScore(student.getObjectiveScore() != null ? student.getObjectiveScore() : 0);
        task.setMarkerScore(student.getTotalScore());
        return task;
    }

    private List<PictureConfigItem> buildPictureConfig(int picCount) {
        List<PictureConfigItem> list = new LinkedList<PictureConfigItem>();
        for (int i = 1; i <= picCount; i++) {
            list.add(new PictureConfigItem(String.valueOf(i)));
        }
        return list;
    }

    private List<MarkStepDTO> buildMarkStep(ExamStudent student) {
        List<MarkStepDTO> list = new LinkedList<MarkStepDTO>();
        List<ExamQuestion> sList = questionService.findByExamAndSubjectAndObjective(student.getExamId(),
                student.getSubjectCode(), false);
        List<ScoreItem> sItems = student.getScoreList(false);
        for (int i = 0; i < sList.size(); i++) {
            ExamQuestion question = sList.get(i);
            MarkStepDTO step = buildStep(question);
            if (!sItems.isEmpty() && sItems.size() == sList.size()) {
                step.setScore(sItems.get(i).getScore());
            }
            // 增加阅卷轨迹列表获取
            List<MarkTrack> tracks = trackService.findByStudentId(student.getId());
            String questionNumber = question.getQuestionNumber();
            for (MarkTrack track : tracks) {
                if (track.getQuestionNumber().equals(questionNumber)) {
                    step.addTrack(new TrackDTO(track));
                }
            }
            list.add(step);
        }
        return list;
    }

}
