package cn.com.qmth.stmms.mark;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerClassService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.MarkResult;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.biz.mark.model.ProblemType;
import cn.com.qmth.stmms.biz.mark.model.SubmitResult;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.model.TrialHistory;
import cn.com.qmth.stmms.biz.mark.model.TrialLibrary;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.ProblemTypeService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.biz.mark.service.TrialService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.controller.BaseController;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.ExamType;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.MarkMode;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

@Controller
public class MarkApi extends BaseController {

    private static Logger log = LoggerFactory.getLogger(MarkApi.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private TrialService trialService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private MarkService markService;

    @Autowired
    private ExamService examService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private LockService lockService;

    @Autowired
    private ProblemTypeService problemTypeService;

    @Autowired
    private UserService userService;

    @Autowired
    private MarkerClassService markerClassService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private FileService fileService;

    @Value("${marker.forceMode}")
    private String forceMarkMode;

    @Value("${slice.split.config}")
    private String splitConfig;

    @Value("${marker.prefetch.count}")
    private long prefetchCount;

    @RequestMapping(value = "/mark/getSetting", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject getSetting(HttpServletRequest request) {
        JSONObject setting = new JSONObject();
        WebUser wu = RequestUtils.getWebUser(request);
        Marker marker = wu.getMarker();
        Exam exam = examService.findById(marker.getExamId());
        MarkGroup group = groupService.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        setting.accumulate("examType", exam.getType());
        setting.accumulate("startTime", exam.getStartTime().getTime());
        setting.accumulate("endTime", exam.getEndTime().getTime());
        setting.accumulate("topCount", marker.getTopCount() != null ? marker.getTopCount() : 0);
        setting.accumulate("sheetView", group.isSheetView());
        setting.accumulate("enableAllZero", group.isEnableAllZero());
        setting.accumulate("statusValue", group.getStatus());
        setting.accumulate("groupNumber", group.getNumber());
        setting.accumulate("fileServer", fileService.getFileServer());
        setting.accumulate("forceSpecialTag", exam.isForceSpecialTag());
        setting.accumulate("uiSetting",
                StringUtils.isBlank(marker.getMarkSetting()) ? new JSONObject() : marker.getMarkSetting());
        setting.accumulate("splitConfig", getSplitConfig());
        setting.accumulate("prefetchCount", prefetchCount);
        setting.accumulate("returnUrl", wu.getLogoutUrl() == null ? "mark-login" : wu.getLogoutUrl());
        setModeAndForceMode(setting, exam, marker, group);
        setSubjectAndSheetConfig(setting, marker, exam);
        setMarker(setting, marker, RequestUtils.getWebUser(request).getName());
        setProblemType(setting, marker);
        return setting;
    }

    private double[] getSplitConfig() {
        String strs[] = splitConfig.split(",");
        double[] config = new double[strs.length];
        for (int i = 0; i < strs.length; i++) {
            config[i] = Double.parseDouble(strs[i]);
        }
        return config;
    }

    @RequestMapping(value = "/mark/getStatus", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject status(HttpServletRequest request) {
        JSONObject status = new JSONObject();
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        ExamSubject subject = subjectService.find(marker.getExamId(), marker.getSubjectCode());
        MarkGroup group = groupService.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        if (subject == null || group == null || group.getStatus() == MarkStatus.FINISH) {
            status.accumulate("valid", false);
            return status;
        }
        long totalCount = 0;
        long personCount = 0;
        long markedCount = 0;
        long problemCount = 0;
        long arbitrateCount = 0;
        if (group.getStatus() == MarkStatus.FORMAL) {
            MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
            query.setExamId(marker.getExamId());
            query.setSubjectCode(marker.getSubjectCode());
            query.setGroupNumber(marker.getGroupNumber());
            totalCount = libraryService.countByQuery(query);

            query.setMarkerId(marker.getId());
            personCount = libraryService.countByQuery(query);

            query.setMarkerId(0);
            query.addStatus(LibraryStatus.MARKED);
            query.addStatus(LibraryStatus.ARBITRATED);
            query.addStatus(LibraryStatus.INSPECTED);
            markedCount = libraryService.countByQuery(query);

            query.clearStatus();
            query.addStatus(LibraryStatus.PROBLEM);
            problemCount = libraryService.countByQuery(query);

            query.clearStatus();
            query.addStatus(LibraryStatus.WAIT_ARBITRATE);
            arbitrateCount = libraryService.countByQuery(query);
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            totalCount = trialService.countLibrary(group.getExamId(), group.getSubjectCode(), group.getNumber());
            personCount = trialService.countMarkerHistory(marker.getId());
            markedCount = personCount;
        }
        status.accumulate("personCount", personCount);
        status.accumulate("totalCount", totalCount);
        status.accumulate("markedCount", markedCount);
        status.accumulate("problemCount", problemCount);
        status.accumulate("arbitrateCount", arbitrateCount);
        status.accumulate("valid", totalCount > 0);
        return status;
    }

    @RequestMapping(value = "/mark/getGroup", method = RequestMethod.POST)
    @ResponseBody
    public JSONArray getGroup(HttpServletRequest request) {
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        JSONArray array = new JSONArray();
        List<Marker> markers = markerService.findByExamAndSubjectAndUserIdAndEnable(marker.getExamId(),
                marker.getSubjectCode(), marker.getUserId(), true);
        for (Marker m : markers) {
            JSONObject json = new JSONObject();
            json.accumulate("markerId", m.getId());
            json.accumulate("number", m.getGroupNumber());
            MarkGroup group = groupService
                    .findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(marker.getExamId(),
                    marker.getSubjectCode(), false, group.getNumber()));
            json.accumulate("title", group.getTitle());
            json.accumulate("markedCount", group.getMarkerCount());
            json.accumulate("totalCount", group.getLibraryCount());
            array.add(json);
        }
        return array;
    }

    @RequestMapping(value = "/mark/getTask", method = RequestMethod.POST)
    @ResponseBody
    public Task getTask(HttpServletRequest request) {
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        Task task = null;
        try {
            lockService.watch(LockType.GROUP, marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            lockService.watch(LockType.MARKER, marker.getId());

            MarkGroup group = groupService
                    .findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            if (group == null) {
                task = new Task();
                // task.setExist(false);
                // task.setMessage("mark.control.task.not.exist");
            } else if (group.getStatus() == MarkStatus.FINISH) {
                task = new Task();
                // task.setExist(false);
                // task.setMessage("mark.control.task.finish");
            } else if (group.getStatus() == MarkStatus.TRIAL) {
                task = getTrialTask(marker);
            } else if (group.getStatus() == MarkStatus.FORMAL) {
                task = getFormalTask(marker);
            }
            if (task == null) {
                task = new Task();
                // task.setExist(false);
                // task.setMessage("mark.control.task.null");
            }
        } catch (Exception e) {
            log.error("get task error", e);
        } finally {
            lockService.unwatch(LockType.MARKER, marker.getId());
            lockService.unwatch(LockType.GROUP, marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        }
        return task;
    }

    private Task getFormalTask(Marker marker) {
        int retry = 1;
        Task task = null;
        while (task == null) {
            List<MarkLibrary> list = new ArrayList<MarkLibrary>();
            // 需要判断评卷员是否绑定了班级
            long classCount = markerClassService.countByUserIdAndExamId(marker.getUserId(), marker.getExamId());
            list = libraryService.findUnMarked(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber(),
                    marker.getId(), marker.getUserId(), classCount > 0, retry, 20);
            if (list.isEmpty()) {
                break;
            }
            for (MarkLibrary library : list) {
                if (markService.applyLibrary(library, marker)) {
                    task = taskService.build(library);
                    break;
                }
            }
            if (task == null) {
                retry++;
            }
        }
        return task;
    }

    private Task getTrialTask(Marker marker) {
        int retry = 1;
        Task task = null;
        while (task == null) {
            List<TrialLibrary> list = trialService.findUnMarkedLibrary(marker.getExamId(), marker.getSubjectCode(),
                    marker.getGroupNumber(), marker.getId(), retry, 10);
            if (list.isEmpty()) {
                break;
            }
            for (TrialLibrary library : list) {
                if (markService.applyLibrary(library, marker)) {
                    task = taskService.build(library, null);
                    break;
                }
            }
            retry++;
        }
        return task;
    }

    @Logging(menu = "评卷", type = LogType.UPDATE)
    @RequestMapping(value = "/mark/saveTask", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject saveTask(HttpServletRequest request, @RequestBody MarkResult markResult) {
        JSONObject result = new JSONObject();
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        boolean success = false;
        try {
            lockService.watch(LockType.EXAM_SUBJECT, marker.getExamId(), marker.getSubjectCode());
            lockService.watch(LockType.GROUP, marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            lockService.watch(LockType.MARKER, marker.getId());
            lockService.waitlock(LockType.STUDENT, markResult.getStudentId());
            SubmitResult sr = markService.submitTask(markResult, marker);
            if (sr.isSuccess()) {
                markService.releaseTask(sr, marker);
                success = true;
            }
        } catch (Exception e) {
            log.error("save task error", e);
        } finally {
            lockService.unlock(LockType.STUDENT, markResult.getStudentId());
            lockService.unwatch(LockType.MARKER, marker.getId());
            lockService.unwatch(LockType.GROUP, marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
            lockService.unwatch(LockType.EXAM_SUBJECT, marker.getExamId(), marker.getSubjectCode());
        }
        result.accumulate("success", success);
        result.accumulate("status", status(request));
        if (!success) {
            result.accumulate("message", "mark.control.task.error");
        }
        return result;
    }

    @Logging(menu = "查询回评任务", type = LogType.QUERY)
    @RequestMapping(value = "/mark/getHistory", method = RequestMethod.POST)
    @ResponseBody
    public Object history(HttpServletRequest request, @RequestParam int pageNumber, @RequestParam int pageSize,
            @RequestParam String order, @RequestParam String sort, @RequestParam(required = false) String secretNumber)
            throws Exception {
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        List<Task> list = new ArrayList<>();
        Direction d = Direction.DESC;
        Sort querySort = null;
        if (sort.equals("ASC")) {
            d = Direction.ASC;
        }
        if (order.equals("markerTime")) {
            querySort = new Sort(d, "markerTime");
        } else if (order.equals("markerScore")) {
            querySort = new Sort(d, "markerScore");
        }
        MarkGroup group = groupService.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        if (group != null && group.getStatus() == MarkStatus.FORMAL) {
            // 正评查找已给分的评卷任务
            MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
            query.setExamId(marker.getExamId());
            query.setSubjectCode(marker.getSubjectCode());
            query.setMarkerId(marker.getId());
            query.addStatus(LibraryStatus.MARKED);
            query.addStatus(LibraryStatus.INSPECTED);
            query.setGroupNumber(marker.getGroupNumber());
            query.setSecretNumber(secretNumber);
            query.setPageNumber(pageNumber);
            query.setPageSize(pageSize);
            if (querySort != null) {
                query.setSort(querySort);
            } else {
                query.orderByMarkerTimeDesc();
            }
            list = taskService.findByQuery(query);
            for (Task task : list) {
                task.setPrevious(true);
            }
        } else if (group != null && group.getStatus() == MarkStatus.TRIAL) {
            // 试评查找给分历史记录
            List<TrialHistory> historyList = new ArrayList<TrialHistory>();
            if (StringUtils.isNotBlank(secretNumber)) {
                historyList = trialService.findHistory(marker.getExamId(), marker.getSubjectCode(),
                        marker.getGroupNumber(), marker.getId(), secretNumber, pageNumber, pageSize, querySort);
            } else {
                historyList = trialService.findHistory(marker.getExamId(), marker.getSubjectCode(),
                        marker.getGroupNumber(), marker.getId(), pageNumber, pageSize, querySort, null);
            }
            for (TrialHistory history : historyList) {
                TrialLibrary library = trialService.findLibrary(history.getLibraryId());
                if (library != null) {
                    Task task = taskService.build(library, history);
                    task.setPrevious(true);
                    list.add(task);
                }
            }
        }
        return list;
    }

    @Logging(menu = "修改密码", type = LogType.UPDATE)
    @RequestMapping(value = "/mark/changeName", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject changeName(HttpServletRequest request, @RequestParam String name,
            @RequestParam(required = false) String password) {
        User user = userService.findById(RequestUtils.getWebUser(request).getId());
        JSONObject result = new JSONObject();
        user.setName(name);
        if (StringUtils.isNotEmpty(password)) {
            user.setPassword(EncryptUtils.md5(password));
        }
        try {
            user = userService.save(user);
            result.accumulate("success", true);
        } catch (Exception e) {
            result.accumulate("success", false);
            log.error("MarkController-修改名字出错", e);
        }
        return result;
    }

    @RequestMapping(value = "/mark/updateSetting", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject updateSetting(HttpServletRequest request, @RequestParam(required = false) String uiSetting,
            @RequestParam(required = false) String mode) {
        Marker marker = RequestUtils.getWebUser(request).getMarker();
        if (uiSetting != null) {
            markerService.updateMarkSetting(marker.getId(),
                    StringEscapeUtils.unescapeHtml(StringUtils.trimToNull(uiSetting)));
        }
        if (mode != null) {
            marker.setMode(MarkMode.findByName(mode));
            markerService.save(marker);
        }
        JSONObject result = new JSONObject();
        result.accumulate("success", true);
        return result;
    }

    @RequestMapping("/mark/clear")
    @ResponseBody
    public void clear(HttpServletRequest request) {
        releaseMarker(RequestUtils.getWebUser(request).getMarker());
    }

    private void releaseMarker(Marker marker) {
        if (marker == null) {
            return;
        }
        try {
            lockService.waitlock(LockType.MARKER, marker.getId());
            markService.releaseByMarker(marker);
        } catch (Exception e) {
            log.error("release marker error", e);
        } finally {
            lockService.unlock(LockType.MARKER, marker.getId());
        }
    }

    protected String buildPictureConfig(String sheetConfig) {
        String json = "";
        if (StringUtils.isNotBlank(sheetConfig)) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                json = mapper.writeValueAsString(PictureConfigItem.parse(sheetConfig));
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
        }
        return json;
    }

    private void setProblemType(JSONObject setting, Marker marker) {
        List<ProblemType> problemTypes = problemTypeService.findByExamId(marker.getExamId());
        ObjectMapper mapper = new ObjectMapper();
        try {
            setting.accumulate("problemTypes", mapper.writeValueAsString(problemTypes));
        } catch (JsonProcessingException e) {
            log.error("MarkController-问题类型获取出错", e);
        }
    }

    private void setMarker(JSONObject setting, Marker marker, String name) {
        JSONObject m = new JSONObject();
        m.accumulate("id", marker.getId());
        m.accumulate("name", name);
        setting.accumulate("marker", m);
    }

    private void setSubjectAndSheetConfig(JSONObject setting, Marker marker, Exam exam) {
        ExamSubject examSubject = subjectService.find(marker.getExamId(), marker.getSubjectCode());
        String sheetConfig = "";
        if (StringUtils.isNotBlank(examSubject.getSheetConfig())) {
            sheetConfig = buildPictureConfig(examSubject.getSheetConfig());
        } else if (StringUtils.isNotBlank(exam.getSheetConfig())) {
            sheetConfig = buildPictureConfig(exam.getSheetConfig());
        }
        setting.accumulate("sheetConfig", sheetConfig);

        JSONObject subject = new JSONObject();
        subject.accumulate("name", examSubject.getName());
        subject.accumulate("code", examSubject.getCode());
        subject.accumulate("paperUrl", examSubject.getPaperUrl() == null ? "" : examSubject.getPaperUrl());
        subject.accumulate("answerUrl", examSubject.getAnswerUrl() == null ? "" : examSubject.getAnswerUrl());
        setting.accumulate("subject", subject);
    }

    private void setModeAndForceMode(JSONObject setting, Exam exam, Marker marker, MarkGroup group) {
        MarkMode mode = null;
        boolean forceMode = false;
        if (MarkMode.findByName(forceMarkMode) != null) {
            // 全局配置的强制评卷模式
            mode = MarkMode.findByName(forceMarkMode);
            forceMode = true;
        } else {
            // 没有全局配置，优先从大题配置取强制评卷模式
            if (group != null && group.getMarkMode() != null) {
                mode = group.getMarkMode();
                forceMode = true;
            }
            // 否则取评卷员当前记录的评卷模式
            if (mode == null) {
                mode = marker.getMode();
            }
            if (mode == null) {
                mode = MarkMode.COMMON;
            }
            if (marker.getMode() != mode) {
                marker.setMode(mode);
                markerService.save(marker);
            }
        }
        setting.accumulate("mode", mode);
        setting.accumulate("forceMode", forceMode);

        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            setting.accumulate("forceMode", true);
            setting.accumulate("sheetView", false);
            setting.accumulate("forceSpecialTag", false);
        }
    }

}
